@@ -55,9 +55,11 @@ public:
 
 	ButtonStyle get_button_style() const;
 	static std::string get_button_title_default();
+	void get_menu_position(int* x, int* y) const;
 
-	void menu_hidden(bool lost_focus);
-	void reload();
+	void menu_hidden();
+	void reload_button();
+	void reload_menu();
 	void set_button_style(ButtonStyle style);
 	void set_button_title(const std::string& title);
 	void set_button_icon_name(const std::string& icon);
@@ -65,16 +67,14 @@ public:
 	void set_loaded(bool loaded);
 
 private:
-	void button_toggled(GtkToggleButton* button);
 	void configure();
 	void icon_changed(const gchar* icon);
 	void mode_changed(XfcePanelPluginMode mode);
 	gboolean remote_event(const gchar* name, const GValue* value);
-	void save();
 	void show_about();
 	gboolean size_changed(gint size);
 	void update_size();
-	void show_menu(bool at_cursor);
+	void show_menu(int position);
 
 private:
 	XfcePanelPlugin* m_plugin;
@@ -87,7 +87,7 @@ private:
 
 	int m_opacity;
 	bool m_file_icon;
-	bool m_menu_shown;
+	gint64 m_hide_time;
 };
 
 }
