@@ -94,8 +94,12 @@ Profile::Profile(Window* window) :
 Profile::~Profile()
 {
 #ifdef HAVE_ACCOUNTS_SERVICE
+	if (m_act_user)
+	{
+		g_object_unref(m_act_user);
+	}
+
 	g_object_unref(m_act_user_manager);
-	g_object_unref(m_act_user);
 #endif
 
 	if (m_file_monitor)
@@ -234,6 +238,7 @@ void Profile::on_user_info_loaded()
 		});
 
 	m_act_user = act_user_manager_get_user_by_id(m_act_user_manager, getuid());
+	g_object_ref(m_act_user);
 	if (act_user_is_loaded(m_act_user))
 	{
 		on_user_changed(m_act_user);
