@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2021 Graeme Gott <graeme@gottcode.org>
+ * Copyright (C) 2013-2023 Graeme Gott <graeme@gottcode.org>
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -77,7 +77,6 @@ SettingsDialog::SettingsDialog(Plugin* plugin) :
 	m_plugin(plugin)
 {
 	// Create dialog window
-#if LIBXFCE4UI_CHECK_VERSION(4,13,0)
 	m_window = xfce_titled_dialog_new_with_mixed_buttons(_("Whisker Menu"),
 			nullptr,
 			GtkDialogFlags(0),
@@ -85,14 +84,6 @@ SettingsDialog::SettingsDialog(Plugin* plugin) :
 			"window-close-symbolic", _("_Close"), GTK_RESPONSE_CLOSE,
 			nullptr);
 	gtk_window_set_type_hint(GTK_WINDOW(m_window), GDK_WINDOW_TYPE_HINT_NORMAL);
-#else
-	m_window = xfce_titled_dialog_new_with_buttons(_("Whisker Menu"),
-			GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(m_plugin->get_button()))),
-			GtkDialogFlags(0),
-			_("_Help"), GTK_RESPONSE_HELP,
-			_("_Close"), GTK_RESPONSE_CLOSE,
-			nullptr);
-#endif
 	gtk_window_set_icon_name(GTK_WINDOW(m_window), "org.xfce.panel.whiskermenu");
 	gtk_window_set_position(GTK_WINDOW(m_window), GTK_WIN_POS_CENTER);
 
@@ -395,7 +386,7 @@ GtkWidget* SettingsDialog::init_general_tab()
 			if (gtk_toggle_button_get_active(button))
 			{
 				wm_settings->view_mode = Settings::ViewAsIcons;
-				m_plugin->reload();
+				m_plugin->reload_menu();
 
 				gtk_widget_set_sensitive(m_show_descriptions, false);
 			}
@@ -407,7 +398,7 @@ GtkWidget* SettingsDialog::init_general_tab()
 			if (gtk_toggle_button_get_active(button))
 			{
 				wm_settings->view_mode = Settings::ViewAsList;
-				m_plugin->reload();
+				m_plugin->reload_menu();
 
 				gtk_widget_set_sensitive(m_show_descriptions, true);
 			}
@@ -419,7 +410,7 @@ GtkWidget* SettingsDialog::init_general_tab()
 			if (gtk_toggle_button_get_active(button))
 			{
 				wm_settings->view_mode = Settings::ViewAsTree;
-				m_plugin->reload();
+				m_plugin->reload_menu();
 
 				gtk_widget_set_sensitive(m_show_descriptions, true);
 			}
@@ -438,7 +429,7 @@ GtkWidget* SettingsDialog::init_general_tab()
 		[this](GtkToggleButton* button)
 		{
 			wm_settings->launcher_show_name = !gtk_toggle_button_get_active(button);
-			m_plugin->reload();
+			m_plugin->reload_menu();
 		});
 
 	// Add option to hide category names
@@ -474,7 +465,7 @@ GtkWidget* SettingsDialog::init_general_tab()
 		[this](GtkToggleButton* button)
 		{
 			wm_settings->launcher_show_description = gtk_toggle_button_get_active(button);
-			m_plugin->reload();
+			m_plugin->reload_menu();
 		});
 
 	// Add space beneath options
@@ -538,14 +529,50 @@ GtkWidget* SettingsDialog::init_general_tab()
 	gtk_widget_set_margin_bottom(m_category_icon_size, 12);
 
 
-	// Add option to control background opacity
-	label = gtk_label_new_with_mnemonic(_("Background opacit_y:"));
+	// Add option to change menu width
+	label = gtk_label_new_with_mnemonic(_("Menu _width:"));
 	gtk_widget_set_halign(label, GTK_ALIGN_START);
 	gtk_grid_attach(page, label, 0, 7, 1, 1);
 
+	m_menu_width = gtk_spin_button_new_with_range(100, 10000, 1);
+	gtk_widget_set_halign(m_menu_width, GTK_ALIGN_START);
+	gtk_widget_set_hexpand(m_menu_width, false);
+	gtk_grid_attach(page, m_menu_width, 1, 7, 1, 1);
+	gtk_label_set_mnemonic_widget(GTK_LABEL(label), m_menu_width);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_menu_width), wm_settings->menu_width);
+
+	connect(m_menu_width, "value-changed",
+		[](GtkSpinButton* button)
+		{
+			wm_settings->menu_width = gtk_spin_button_get_value_as_int(button);
+		});
+
+	// Add option to change menu height
+	label = gtk_label_new_with_mnemonic(_("Menu _height:"));
+	gtk_widget_set_halign(label, GTK_ALIGN_START);
+	gtk_grid_attach(page, label, 0, 8, 1, 1);
+
+	m_menu_height = gtk_spin_button_new_with_range(100, 10000, 1);
+	gtk_widget_set_halign(m_menu_height, GTK_ALIGN_START);
+	gtk_widget_set_hexpand(m_menu_height, false);
+	gtk_grid_attach(page, m_menu_height, 1, 8, 1, 1);
+	gtk_label_set_mnemonic_widget(GTK_LABEL(label), m_menu_height);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_menu_height), wm_settings->menu_height);
+
+	connect(m_menu_height, "value-changed",
+		[](GtkSpinButton* button)
+		{
+			wm_settings->menu_height = gtk_spin_button_get_value_as_int(button);
+		});
+
+	// Add option to control background opacity
+	label = gtk_label_new_with_mnemonic(_("Background opacit_y:"));
+	gtk_widget_set_halign(label, GTK_ALIGN_START);
+	gtk_grid_attach(page, label, 0, 9, 1, 1);
+
 	m_background_opacity = gtk_scale_new_with_range(GTK_ORIENTATION_HORIZONTAL, 0.0, 100.0, 1.0);
 	gtk_widget_set_hexpand(GTK_WIDGET(m_background_opacity), true);
-	gtk_grid_attach(page, m_background_opacity, 1, 7, 1, 1);
+	gtk_grid_attach(page, m_background_opacity, 1, 9, 1, 1);
 	gtk_scale_set_value_pos(GTK_SCALE(m_background_opacity), GTK_POS_RIGHT);
 	gtk_range_set_value(GTK_RANGE(m_background_opacity), wm_settings->menu_opacity);
 
@@ -596,10 +623,13 @@ GtkWidget* SettingsDialog::init_appearance_tab()
 			wm_settings->position_categories_horizontal = gtk_toggle_button_get_active(button);
 			const bool active = (wm_settings->category_icon_size != -1) && !wm_settings->position_categories_horizontal;
 			gtk_widget_set_sensitive(m_show_category_names, active);
+			gtk_button_set_label(GTK_BUTTON(m_position_categories_alternate),
+					wm_settings->position_categories_horizontal ? _("Position cate_gories on bottom") : _("Position cate_gories on left"));
 		});
 
 	// Add option to use alternate categories position
-	m_position_categories_alternate = gtk_check_button_new_with_mnemonic(_("Position cate_gories next to panel button"));
+	m_position_categories_alternate = gtk_check_button_new_with_mnemonic(
+			wm_settings->position_categories_horizontal ? _("Position cate_gories on bottom") : _("Position cate_gories on left"));
 	gtk_grid_attach(menu_table, m_position_categories_alternate, 0, 1, 2, 1);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_position_categories_alternate), wm_settings->position_categories_alternate);
 
@@ -609,9 +639,20 @@ GtkWidget* SettingsDialog::init_appearance_tab()
 			wm_settings->position_categories_alternate = gtk_toggle_button_get_active(button);
 		});
 
+	// Add option to use alternate profile position
+	m_position_profile_alternate = gtk_check_button_new_with_mnemonic(_("Position pro_file on bottom"));
+	gtk_grid_attach(menu_table, m_position_profile_alternate, 0, 2, 2, 1);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_position_profile_alternate), wm_settings->position_profile_alternate);
+
+	connect(m_position_profile_alternate, "toggled",
+		[](GtkToggleButton* button)
+		{
+			wm_settings->position_profile_alternate = gtk_toggle_button_get_active(button);
+		});
+
 	// Add option to use alternate search entry position
-	m_position_search_alternate = gtk_check_button_new_with_mnemonic(_("Position _search entry next to panel button"));
-	gtk_grid_attach(menu_table, m_position_search_alternate, 0, 2, 2, 1);
+	m_position_search_alternate = gtk_check_button_new_with_mnemonic(_("Position _search entry on bottom"));
+	gtk_grid_attach(menu_table, m_position_search_alternate, 0, 3, 2, 1);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_position_search_alternate), wm_settings->position_search_alternate);
 
 	connect(m_position_search_alternate, "toggled",
@@ -622,7 +663,7 @@ GtkWidget* SettingsDialog::init_appearance_tab()
 
 	// Add option to use alternate commands position
 	m_position_commands_alternate = gtk_check_button_new_with_mnemonic(_("Position commands next to search _entry"));
-	gtk_grid_attach(menu_table, m_position_commands_alternate, 0, 3, 2, 1);
+	gtk_grid_attach(menu_table, m_position_commands_alternate, 0, 4, 2, 1);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_position_commands_alternate), wm_settings->position_commands_alternate);
 
 	connect(m_position_commands_alternate, "toggled",
@@ -631,11 +672,10 @@ GtkWidget* SettingsDialog::init_appearance_tab()
 			wm_settings->position_commands_alternate = gtk_toggle_button_get_active(button);
 		});
 
-
 	// Add profile shape selector
 	GtkWidget* label = gtk_label_new_with_mnemonic(_("P_rofile:"));
 	gtk_widget_set_halign(label, GTK_ALIGN_START);
-	gtk_grid_attach(menu_table, label, 0, 4, 1, 1);
+	gtk_grid_attach(menu_table, label, 0, 5, 1, 1);
 
 	m_profile_shape = gtk_combo_box_text_new();
 	gtk_widget_set_halign(m_profile_shape, GTK_ALIGN_START);
@@ -644,7 +684,7 @@ GtkWidget* SettingsDialog::init_appearance_tab()
 	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(m_profile_shape), _("Square Picture"));
 	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(m_profile_shape), _("Hidden"));
 	gtk_combo_box_set_active(GTK_COMBO_BOX(m_profile_shape), wm_settings->profile_shape);
-	gtk_grid_attach(menu_table, m_profile_shape, 1, 4, 1, 1);
+	gtk_grid_attach(menu_table, m_profile_shape, 1, 5, 1, 1);
 	gtk_label_set_mnemonic_widget(GTK_LABEL(label), m_profile_shape);
 
 	connect(m_profile_shape, "changed",
@@ -848,7 +888,7 @@ GtkWidget* SettingsDialog::init_behavior_tab()
 		[this](GtkToggleButton* button)
 		{
 			wm_settings->sort_categories = gtk_toggle_button_get_active(button);
-			m_plugin->reload();
+			m_plugin->reload_menu();
 		});
 
 
