@@ -25,7 +25,6 @@
 #include "plugin.h"
 #include "profile.h"
 #include "recent-page.h"
-#include "resizer.h"
 #include "search-page.h"
 #include "settings.h"
 #include "slot.h"
@@ -34,6 +33,10 @@
 #include <gdk/gdkkeysyms.h>
 #include <libxfce4ui/libxfce4ui.h>
 
+#ifdef HAVE_GTK_LAYER_SHELL
+#include <gtk-layer-shell.h>
+#endif
+
 #include <ctime>
 
 using namespace WhiskerMenu;
@@ -44,13 +47,13 @@ WhiskerMenu::Window::Window(Plugin* plugin) :
 	m_plugin(plugin),
 	m_window(nullptr),
 	m_sidebar_size_group(nullptr),
-	m_geometry{0,0,wm_settings->menu_width,wm_settings->menu_height},
-	m_layout_left(true),
-	m_layout_bottom(true),
+	m_geometry{0,0,1,1},
+	m_layout_ltr(true),
 	m_layout_categories_horizontal(false),
 	m_layout_categories_alternate(false),
 	m_layout_search_alternate(false),
 	m_layout_commands_alternate(false),
+	m_layout_profile_alternate(false),
 	m_profile_shape(0),
 	m_supports_alpha(false),
 	m_child_has_focus(false)
@@ -60,7 +63,12 @@ WhiskerMenu::Window::Window(Plugin* plugin) :
 	gtk_widget_set_name(GTK_WIDGET(m_window), "whiskermenu-window");
 	// Untranslated window title to allow window managers to identify it; not visible to users.
 	gtk_window_set_title(m_window, "Whisker Menu");
-	gtk_window_set_modal(m_window, true);
+#ifdef HAVE_GTK_LAYER_SHELL
+	if (!gtk_layer_is_supported())
+#endif
+	{
+		gtk_window_set_modal(m_window, true);
+	}
 	gtk_window_set_decorated(m_window, false);
 	gtk_window_set_skip_taskbar_hint(m_window, true);
 	gtk_window_set_skip_pager_hint(m_window, true);
@@ -68,6 +76,26 @@ WhiskerMenu::Window::Window(Plugin* plugin) :
 	gtk_window_stick(m_window);
 	gtk_widget_add_events(GTK_WIDGET(m_window), GDK_FOCUS_CHANGE_MASK | GDK_STRUCTURE_MASK);
 
+#ifdef HAVE_GTK_LAYER_SHELL
+	if (gtk_layer_is_supported())
+	{
+		gtk_layer_init_for_window(m_window);
+
+		// Position from top left, and excludes other windows
+		gtk_layer_set_exclusive_zone(m_window, -1);
+		gtk_layer_set_anchor(m_window, GTK_LAYER_SHELL_EDGE_TOP, true);
+		gtk_layer_set_anchor(m_window, GTK_LAYER_SHELL_EDGE_BOTTOM, false);
+		gtk_layer_set_anchor(m_window, GTK_LAYER_SHELL_EDGE_LEFT, true);
+		gtk_layer_set_anchor(m_window, GTK_LAYER_SHELL_EDGE_RIGHT, false);
+
+		// Grab keyboard focus when shown
+		gtk_layer_set_keyboard_mode(m_window, GTK_LAYER_SHELL_KEYBOARD_MODE_EXCLUSIVE);
+
+		// Position menu above other windows
+		gtk_layer_set_layer(m_window, GTK_LAYER_SHELL_LAYER_OVERLAY);
+	}
+#endif
+
 	connect(m_window, "enter-notify-event",
 		[this](GtkWidget*, GdkEvent*) -> gboolean
 		{
@@ -85,10 +113,21 @@ WhiskerMenu::Window::Window(Plugin* plugin) :
 	connect(m_window, "focus-out-event",
 		[this](GtkWidget* widget, GdkEvent*) -> gboolean
 		{
-			if (!wm_settings->stay_on_focus_out && !m_child_has_focus && gtk_widget_get_visible(widget))
+			if (wm_settings->stay_on_focus_out || m_child_has_focus || !gtk_widget_get_visible(widget))
 			{
-				hide(true);
+				return GDK_EVENT_PROPAGATE;
 			}
+
+			// Needed to make focus out event happen after button press event,
+			// otherwise it is impossible to toggle panel button.
+			g_idle_add(
+				+[](gpointer user_data) -> gboolean
+				{
+					static_cast<Window*>(user_data)->hide(true);
+					return G_SOURCE_REMOVE;
+				},
+			this);
+
 			return GDK_EVENT_PROPAGATE;
 		});
 
@@ -117,18 +156,6 @@ WhiskerMenu::Window::Window(Plugin* plugin) :
 			on_state_flags_changed(widget);
 		});
 
-	connect(m_window, "configure-event",
-		[this](GtkWidget*, GdkEvent* event) -> gboolean
-		{
-			return on_configure_event(reinterpret_cast<GdkEventConfigure*>(event));
-		});
-
-	connect(m_window, "window-state-event",
-		[this](GtkWidget*, GdkEvent* event) -> gboolean
-		{
-			return on_window_state_event(reinterpret_cast<GdkEventWindowState*>(event));
-		});
-
 	g_signal_connect(G_OBJECT(m_window), "delete-event", G_CALLBACK(&gtk_widget_hide_on_delete), nullptr);
 
 	// Create the border of the window
@@ -146,23 +173,6 @@ WhiskerMenu::Window::Window(Plugin* plugin) :
 	gtk_widget_set_valign(GTK_WIDGET(m_window_load_spinner), GTK_ALIGN_CENTER);
 	gtk_stack_add_named(m_window_stack, GTK_WIDGET(m_window_load_spinner), "load");
 
-	// Create resizers
-	m_resize[Resizer::TopLeft] = new Resizer(Resizer::TopLeft, this);
-	m_resize[Resizer::Top] = new Resizer(Resizer::Top, this);
-	m_resize[Resizer::TopRight] = new Resizer(Resizer::TopRight, this);
-	m_resize[Resizer::Left] = new Resizer(Resizer::Left, this);
-	m_resize[Resizer::Right] = new Resizer(Resizer::Right, this);
-	m_resize[Resizer::BottomLeft] = new Resizer(Resizer::BottomLeft, this);
-	m_resize[Resizer::Bottom] = new Resizer(Resizer::Bottom, this);
-	m_resize[Resizer::BottomRight] = new Resizer(Resizer::BottomRight, this);
-	for (int i = 0; i < 8; ++i)
-	{
-		gtk_widget_set_sensitive(m_resize[i]->get_widget(), false);
-	}
-	gtk_widget_set_sensitive(m_resize[Resizer::Top]->get_widget(), true);
-	gtk_widget_set_sensitive(m_resize[Resizer::TopRight]->get_widget(), true);
-	gtk_widget_set_sensitive(m_resize[Resizer::Right]->get_widget(), true);
-
 	// Create the profile picture and username label
 	m_profile = new Profile(this);
 
@@ -228,22 +238,10 @@ WhiskerMenu::Window::Window(Plugin* plugin) :
 	// Create search results
 	m_search_results = new SearchPage(this);
 
-	// Create grid for packing resizers
-	GtkGrid* grid = GTK_GRID(gtk_grid_new());
-	gtk_grid_attach(grid, m_resize[Resizer::TopLeft]->get_widget(), 0, 0, 1, 1);
-	gtk_grid_attach(grid, m_resize[Resizer::Top]->get_widget(), 1, 0, 1, 1);
-	gtk_grid_attach(grid, m_resize[Resizer::TopRight]->get_widget(), 2, 0, 1, 1);
-	gtk_grid_attach(grid, m_resize[Resizer::Left]->get_widget(), 0, 1, 1, 1);
-	gtk_grid_attach(grid, m_resize[Resizer::Right]->get_widget(), 2, 1, 1, 1);
-	gtk_grid_attach(grid, m_resize[Resizer::BottomLeft]->get_widget(), 0, 2, 1, 1);
-	gtk_grid_attach(grid, m_resize[Resizer::Bottom]->get_widget(), 1, 2, 1, 1);
-	gtk_grid_attach(grid, m_resize[Resizer::BottomRight]->get_widget(), 2, 2, 1, 1);
-	gtk_stack_add_named(m_window_stack, GTK_WIDGET(grid), "contents");
-
 	// Create box for packing children
 	m_vbox = GTK_BOX(gtk_box_new(GTK_ORIENTATION_VERTICAL, 6));
-	gtk_container_set_border_width(GTK_CONTAINER(m_vbox), 0);
-	gtk_grid_attach(grid, GTK_WIDGET(m_vbox), 1, 1, 1, 1);
+	gtk_container_set_border_width(GTK_CONTAINER(m_vbox), 6);
+	gtk_stack_add_named(m_window_stack, GTK_WIDGET(m_vbox), "contents");
 
 	// Create box for packing commands
 	m_commands_box = GTK_BOX(gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0));
@@ -254,7 +252,7 @@ WhiskerMenu::Window::Window(Plugin* plugin) :
 		gtk_box_pack_start(m_commands_box, command, false, false, 0);
 	}
 
-	// Create box for packing username, commands, and resize widget
+	// Create box for packing username and commands
 	m_title_box = GTK_BOX(gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6));
 	gtk_box_pack_start(m_vbox, GTK_WIDGET(m_title_box), false, false, 0);
 	gtk_box_pack_start(m_title_box, m_profile->get_picture(), false, false, 0);
@@ -297,6 +295,7 @@ WhiskerMenu::Window::Window(Plugin* plugin) :
 
 	m_sidebar = GTK_SCROLLED_WINDOW(gtk_scrolled_window_new(nullptr, nullptr));
 	gtk_grid_attach(m_contents_box, GTK_WIDGET(m_sidebar), 1, 1, 1, 1);
+	gtk_scrolled_window_set_propagate_natural_height(m_sidebar, true);
 	gtk_scrolled_window_set_shadow_type(m_sidebar, GTK_SHADOW_NONE);
 	gtk_scrolled_window_set_policy(m_sidebar, GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 	gtk_container_add(GTK_CONTAINER(m_sidebar), GTK_WIDGET(m_category_buttons));
@@ -319,9 +318,6 @@ WhiskerMenu::Window::Window(Plugin* plugin) :
 	gtk_widget_show_all(frame);
 	m_default_button->set_active(true);
 
-	// Resize to last known size
-	gtk_window_set_default_size(m_window, m_geometry.width, m_geometry.height);
-
 	// Handle transparency
 	gtk_widget_set_app_paintable(GTK_WIDGET(m_window), true);
 
@@ -361,11 +357,6 @@ WhiskerMenu::Window::~Window()
 
 	delete m_profile;
 
-	for (int i = 0; i < 8; ++i)
-	{
-		delete m_resize[i];
-	}
-
 	gtk_widget_destroy(GTK_WIDGET(m_window));
 	g_object_unref(m_window);
 }
@@ -374,9 +365,9 @@ WhiskerMenu::Window::~Window()
 
 void WhiskerMenu::Window::hide(bool lost_focus)
 {
-	// Save size
-	wm_settings->menu_width = m_geometry.width;
-	wm_settings->menu_height = m_geometry.height;
+	// Save settings
+	wm_settings->favorites.save();
+	wm_settings->recent.save();
 
 	// Scroll categories to top
 	GtkAdjustment* adjustment = gtk_scrolled_window_get_vadjustment(m_sidebar);
@@ -395,7 +386,10 @@ void WhiskerMenu::Window::hide(bool lost_focus)
 	show_default_page();
 
 	// Inform plugin that window is hidden
-	m_plugin->menu_hidden(lost_focus);
+	if (!lost_focus)
+	{
+		m_plugin->menu_hidden();
+	}
 }
 
 //-----------------------------------------------------------------------------
@@ -470,11 +464,10 @@ void WhiskerMenu::Window::show(const Position position)
 	m_recent->get_view()->reload_icon_size();
 	m_applications->get_view()->reload_icon_size();
 
-	GdkScreen* screen = nullptr;
-	int parent_x = 0, parent_y = 0, parent_w = 0, parent_h = 0;
-	if (position != PositionAtCursor)
+	if (position == PositionAtButton)
 	{
 		// Wait up to half a second for auto-hidden panels to be shown
+		int parent_x = 0, parent_y = 0;
 		clock_t end = clock() + (CLOCKS_PER_SEC / 2);
 		GtkWidget* parent = m_plugin->get_button();
 		GtkWindow* parent_window = GTK_WINDOW(gtk_widget_get_toplevel(parent));
@@ -488,106 +481,83 @@ void WhiskerMenu::Window::show(const Position position)
 			gtk_window_get_position(parent_window, &parent_x, &parent_y);
 		}
 
-		// Fetch parent geometry
-		if (!gtk_widget_get_realized(parent))
-		{
-			gtk_widget_realize(parent);
-		}
-		GdkWindow* window = gtk_widget_get_window(parent);
-		gdk_window_get_origin(window, &parent_x, &parent_y);
-		screen = gdk_window_get_screen(window);
-		parent_w = gdk_window_get_width(window);
-		parent_h = gdk_window_get_height(window);
+		// Fetch position
+		m_plugin->get_menu_position(&m_geometry.x, &m_geometry.y);
 	}
 	else
 	{
+		// Fetch cursor position
 		GdkDisplay* display = gdk_display_get_default();
 		GdkSeat* seat = gdk_display_get_default_seat(display);
 		GdkDevice* device = gdk_seat_get_pointer(seat);
-		gdk_device_get_position(device, &screen, &parent_x, &parent_y);
+		gdk_device_get_position(device, nullptr, &m_geometry.x, &m_geometry.y);
 	}
 
 	// Fetch screen geomtry
 	GdkRectangle monitor;
-	GdkMonitor* monitor_gdk = gdk_display_get_monitor_at_point(gdk_display_get_default(), parent_x, parent_y);
+	GdkMonitor* monitor_gdk = gdk_display_get_monitor_at_point(gdk_display_get_default(), m_geometry.x, m_geometry.y);
 	gdk_monitor_get_geometry(monitor_gdk, &monitor);
 
-	// Prevent window from being larger than screen
-	if (m_geometry.width > monitor.width)
+	// Resize window if necessary, and also prevent it from being larger than screen
+	const int width = std::min(static_cast<int>(wm_settings->menu_width), monitor.width);
+	const int height = std::min(static_cast<int>(wm_settings->menu_height), monitor.height);
+	bool resized = false;
+	if ((m_geometry.width != width) || (m_geometry.height != height))
 	{
-		m_geometry.width = monitor.width;
-		gtk_window_resize(m_window, m_geometry.width, m_geometry.height);
-	}
-	if (m_geometry.height > monitor.height)
-	{
-		m_geometry.height = monitor.height;
-		gtk_window_resize(m_window, m_geometry.width, m_geometry.height);
+		m_geometry.width = width;
+		m_geometry.height = height;
+		gtk_widget_set_size_request(GTK_WIDGET(m_window), m_geometry.width, m_geometry.height);
+		gtk_window_resize(m_window, 1, 1);
+		resized = true;
 	}
 
-	// Find window position
-	bool layout_left = ((2 * (parent_x - monitor.x)) + parent_w) < monitor.width;
-	bool layout_bottom = ((2 * (parent_y - monitor.y)) + (parent_h / 2)) > monitor.height;
-	if (position != PositionVertical)
+	// Center window if requested
+	if (position == PositionAtCenter)
 	{
-		m_geometry.x = layout_left ? parent_x : (parent_x + parent_w - m_geometry.width);
-		m_geometry.y = layout_bottom ? (parent_y - m_geometry.height - 1) : (parent_y + parent_h + 1);
+		m_geometry.x = (monitor.width - m_geometry.width) / 2;
+		m_geometry.y = (monitor.height - m_geometry.height) / 2;
 	}
-	else
-	{
-		m_geometry.x = layout_left ? (parent_x + parent_w + 1) : (parent_x - m_geometry.width - 1);
-		m_geometry.y = layout_bottom ? (parent_y + parent_h - m_geometry.height) : parent_y;
-	}
-
-	// Prevent window from leaving screen
-	m_geometry.x = CLAMP(m_geometry.x, monitor.x, monitor.x + monitor.width - m_geometry.width);
-	m_geometry.y = CLAMP(m_geometry.y, monitor.y, monitor.y + monitor.height - m_geometry.height);
 
 	// Move window
-	gtk_window_move(m_window, m_geometry.x, m_geometry.y);
+	move_window(monitor);
 
 	// Relayout window if necessary
-	if (gtk_widget_get_direction(GTK_WIDGET(m_window)) == GTK_TEXT_DIR_RTL)
-	{
-		layout_left = !layout_left;
-	}
-
-	if ((m_layout_left != layout_left)
-			|| (m_layout_bottom != layout_bottom)
+	const bool layout_ltr = gtk_widget_get_default_direction() != GTK_TEXT_DIR_RTL;
+	if ((m_layout_ltr != layout_ltr)
 			|| (m_layout_categories_horizontal != wm_settings->position_categories_horizontal)
 			|| (m_layout_categories_alternate != wm_settings->position_categories_alternate)
 			|| (m_layout_search_alternate != wm_settings->position_search_alternate)
 			|| (m_layout_commands_alternate != wm_settings->position_commands_alternate)
+			|| (m_layout_profile_alternate != wm_settings->position_profile_alternate)
 			|| (m_profile_shape != wm_settings->profile_shape))
 	{
-		m_layout_left = layout_left;
-		m_layout_bottom = layout_bottom;
+		m_layout_ltr = layout_ltr;
 		m_layout_categories_horizontal = wm_settings->position_categories_horizontal;
 		m_layout_categories_alternate = wm_settings->position_categories_alternate;
 		m_layout_search_alternate = wm_settings->position_search_alternate;
 		m_layout_commands_alternate = wm_settings->position_commands_alternate;
+		m_layout_profile_alternate = wm_settings->position_profile_alternate;
 		m_profile->update_picture();
 		m_profile_shape = wm_settings->profile_shape;
 		update_layout();
 	}
 
-	const bool category_show_name = wm_settings->category_show_name && !wm_settings->position_categories_horizontal;
-	if (!m_sidebar_size_group && m_layout_commands_alternate && category_show_name)
-	{
-		m_sidebar_size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
-		gtk_size_group_add_widget(m_sidebar_size_group, GTK_WIDGET(m_sidebar));
-		gtk_size_group_add_widget(m_sidebar_size_group, GTK_WIDGET(m_commands_box));
-	}
-	else if (m_sidebar_size_group && (!m_layout_commands_alternate || !category_show_name))
-	{
-		gtk_size_group_remove_widget(m_sidebar_size_group, GTK_WIDGET(m_sidebar));
-		gtk_size_group_remove_widget(m_sidebar_size_group, GTK_WIDGET(m_commands_box));
-		g_object_unref(m_sidebar_size_group);
-		m_sidebar_size_group = nullptr;
-	}
-
 	// Show window
 	gtk_window_present(m_window);
-	gtk_window_move(m_window, m_geometry.x, m_geometry.y);
+
+	if (resized)
+	{
+		check_scrollbar_needed();
+	}
+
+	// Fetch position again to make sure window does not overlap panel
+	if (position == PositionAtButton)
+	{
+		m_plugin->get_menu_position(&m_geometry.x, &m_geometry.y);
+	}
+
+	// Move window
+	move_window(monitor);
 }
 
 //-----------------------------------------------------------------------------
@@ -664,10 +634,18 @@ void WhiskerMenu::Window::unset_items()
 
 gboolean WhiskerMenu::Window::on_key_press_event(GtkWidget* widget, GdkEventKey* key_event)
 {
-	// Hide if escape is pressed and there is no text in search entry
-	if ( (key_event->keyval == GDK_KEY_Escape) && xfce_str_is_empty(gtk_entry_get_text(m_search_entry)) )
+	if (key_event->keyval == GDK_KEY_Escape)
 	{
-		hide();
+		// Hide if escape is pressed and there is no text in search entry
+		if (xfce_str_is_empty(gtk_entry_get_text(m_search_entry)))
+		{
+			hide();
+		}
+		// Clear search entry of text if escape is pressed
+		else
+		{
+			gtk_entry_set_text(m_search_entry, "");
+		}
 		return GDK_EVENT_STOP;
 	}
 
@@ -784,37 +762,6 @@ void WhiskerMenu::Window::on_state_flags_changed(GtkWidget* widget)
 
 //-----------------------------------------------------------------------------
 
-gboolean WhiskerMenu::Window::on_configure_event(GdkEventConfigure* configure_event)
-{
-	if (configure_event->width && configure_event->height)
-	{
-		m_geometry.x = configure_event->x;
-		m_geometry.y = configure_event->y;
-		m_geometry.width = configure_event->width;
-		m_geometry.height = configure_event->height;
-	}
-
-	check_scrollbar_needed();
-
-	return GDK_EVENT_PROPAGATE;
-}
-
-//-----------------------------------------------------------------------------
-
-gboolean WhiskerMenu::Window::on_window_state_event(GdkEventWindowState* state_event)
-{
-	// Workaround to detect clicking off menu to hide it; xfwm only sets it to this
-	// state in that case, for all others it is purely GDK_WINDOW_STATE_WITHDRAWN
-	if (state_event->new_window_state == (GDK_WINDOW_STATE_WITHDRAWN | GDK_WINDOW_STATE_STICKY))
-	{
-		m_plugin->menu_hidden(false);
-	}
-
-	return GDK_EVENT_PROPAGATE;
-}
-
-//-----------------------------------------------------------------------------
-
 void WhiskerMenu::Window::on_screen_changed(GtkWidget* widget)
 {
 	GdkScreen* screen = gtk_widget_get_screen(widget);
@@ -873,13 +820,15 @@ gboolean WhiskerMenu::Window::on_draw_event(GtkWidget* widget, cairo_t* cr)
 
 void WhiskerMenu::Window::check_scrollbar_needed()
 {
-	// Find height of sidebar buttons
-	int height = 0;
-	gtk_widget_get_preferred_height(GTK_WIDGET(m_category_buttons), nullptr, &height);
+	// Find height of sidebar and buttons
+	int buttons_height = 0;
+	gtk_widget_get_preferred_height(GTK_WIDGET(m_category_buttons), nullptr, &buttons_height);
+
+	int sidebar_height = 0;
+	gtk_widget_get_preferred_height(GTK_WIDGET(m_sidebar), nullptr, &sidebar_height);
 
 	// Always show scrollbar if sidebar is shorter than buttons
-	int allocated = gtk_widget_get_allocated_height(GTK_WIDGET(m_sidebar));
-	if ((allocated > height) || (allocated == 1))
+	if (sidebar_height >= buttons_height)
 	{
 		gtk_scrolled_window_set_policy(m_sidebar, GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 	}
@@ -918,6 +867,28 @@ void WhiskerMenu::Window::category_toggled()
 
 //-----------------------------------------------------------------------------
 
+void WhiskerMenu::Window::move_window(const GdkRectangle& monitor)
+{
+	// Prevent window from leaving screen
+	m_geometry.x = CLAMP(m_geometry.x, monitor.x, monitor.x + monitor.width - m_geometry.width);
+	m_geometry.y = CLAMP(m_geometry.y, monitor.y, monitor.y + monitor.height - m_geometry.height);
+
+	// Move window
+#ifdef HAVE_GTK_LAYER_SHELL
+	if (gtk_layer_is_supported())
+	{
+		gtk_layer_set_margin(m_window, GTK_LAYER_SHELL_EDGE_LEFT, m_geometry.x);
+		gtk_layer_set_margin(m_window, GTK_LAYER_SHELL_EDGE_TOP, m_geometry.y);
+	}
+	else
+#endif
+	{
+		gtk_window_move(m_window, m_geometry.x, m_geometry.y);
+	}
+}
+
+//-----------------------------------------------------------------------------
+
 void WhiskerMenu::Window::reset_default_button()
 {
 	switch (wm_settings->default_category)
@@ -999,42 +970,6 @@ void WhiskerMenu::Window::search()
 
 void WhiskerMenu::Window::update_layout()
 {
-	// Set enabled resize edges
-	for (int i = 0; i < 8; ++i)
-	{
-		gtk_widget_set_sensitive(m_resize[i]->get_widget(), false);
-	}
-	if (m_layout_left)
-	{
-		if (m_layout_bottom)
-		{
-			gtk_widget_set_sensitive(m_resize[Resizer::Top]->get_widget(), true);
-			gtk_widget_set_sensitive(m_resize[Resizer::TopRight]->get_widget(), true);
-			gtk_widget_set_sensitive(m_resize[Resizer::Right]->get_widget(), true);
-		}
-		else
-		{
-			gtk_widget_set_sensitive(m_resize[Resizer::Bottom]->get_widget(), true);
-			gtk_widget_set_sensitive(m_resize[Resizer::BottomRight]->get_widget(), true);
-			gtk_widget_set_sensitive(m_resize[Resizer::Right]->get_widget(), true);
-		}
-	}
-	else
-	{
-		if (m_layout_bottom)
-		{
-			gtk_widget_set_sensitive(m_resize[Resizer::Top]->get_widget(), true);
-			gtk_widget_set_sensitive(m_resize[Resizer::TopLeft]->get_widget(), true);
-			gtk_widget_set_sensitive(m_resize[Resizer::Left]->get_widget(), true);
-		}
-		else
-		{
-			gtk_widget_set_sensitive(m_resize[Resizer::Bottom]->get_widget(), true);
-			gtk_widget_set_sensitive(m_resize[Resizer::BottomLeft]->get_widget(), true);
-			gtk_widget_set_sensitive(m_resize[Resizer::Left]->get_widget(), true);
-		}
-	}
-
 	// Set vertical position of commands
 	g_object_ref(m_commands_box);
 	gtk_container_remove(GTK_CONTAINER(gtk_widget_get_parent(GTK_WIDGET(m_commands_box))), GTK_WIDGET(m_commands_box));
@@ -1044,7 +979,7 @@ void WhiskerMenu::Window::update_layout()
 
 		if (!m_layout_categories_horizontal)
 		{
-			if (m_layout_left == m_layout_categories_alternate)
+			if (m_layout_ltr == m_layout_categories_alternate)
 			{
 				gtk_box_reorder_child(m_search_box, GTK_WIDGET(m_commands_box), 0);
 				gtk_box_reorder_child(m_search_box, GTK_WIDGET(m_search_entry), 1);
@@ -1052,7 +987,7 @@ void WhiskerMenu::Window::update_layout()
 		}
 		else
 		{
-			if (!m_layout_left)
+			if (!m_layout_ltr)
 			{
 				gtk_box_reorder_child(m_search_box, GTK_WIDGET(m_commands_box), 0);
 				gtk_box_reorder_child(m_search_box, GTK_WIDGET(m_search_entry), 1);
@@ -1106,7 +1041,7 @@ void WhiskerMenu::Window::update_layout()
 	}
 
 	// Arrange horizontal order of profile picture, username, and commands
-	if (m_layout_left && m_layout_commands_alternate)
+	if (m_layout_ltr && m_layout_commands_alternate)
 	{
 		gtk_widget_set_halign(m_profile->get_username(), GTK_ALIGN_START);
 
@@ -1130,7 +1065,7 @@ void WhiskerMenu::Window::update_layout()
 			gtk_box_reorder_child(m_commands_box, m_commands_button[i], 8 - i);
 		}
 	}
-	else if (m_layout_left)
+	else if (m_layout_ltr)
 	{
 		gtk_widget_set_halign(m_profile->get_username(), GTK_ALIGN_START);
 
@@ -1187,17 +1122,17 @@ void WhiskerMenu::Window::update_layout()
 		gtk_grid_set_column_spacing(m_contents_box, 0);
 		gtk_grid_set_row_spacing(m_contents_box, 6);
 
-		gtk_style_context_add_class(context, (m_layout_bottom == m_layout_categories_alternate) ? "bottom" : "top");
+		gtk_style_context_add_class(context, m_layout_categories_alternate ? "bottom" : "top");
 	}
 	else
 	{
 		gtk_grid_set_column_spacing(m_contents_box, 6);
 		gtk_grid_set_row_spacing(m_contents_box, 0);
 
-		gtk_style_context_add_class(context, (m_layout_left == m_layout_categories_alternate) ? "left" : "right");
+		gtk_style_context_add_class(context, (m_layout_ltr == m_layout_categories_alternate) ? "left" : "right");
 	}
 
-	if (m_layout_left != m_layout_categories_alternate)
+	if (m_layout_ltr != m_layout_categories_alternate)
 	{
 		gtk_grid_attach(m_contents_box, GTK_WIDGET(m_panels_stack), 0, 0, 1, 1);
 		gtk_grid_attach(m_contents_box, GTK_WIDGET(m_sidebar), 1, 0, 1, 1);
@@ -1212,7 +1147,7 @@ void WhiskerMenu::Window::update_layout()
 		gtk_box_reorder_child(m_commands_box, m_commands_spacer, 9);
 	}
 
-	if (m_layout_bottom != m_layout_categories_alternate)
+	if (!m_layout_categories_alternate)
 	{
 		gtk_grid_insert_row(m_contents_box, 0);
 		gtk_grid_attach(m_contents_box, GTK_WIDGET(m_categories_box), 0, 0, 2, 1);
@@ -1227,29 +1162,45 @@ void WhiskerMenu::Window::update_layout()
 	g_object_unref(m_categories_box);
 
 	// Arrange vertical order of header, applications, and search
-	if (m_layout_bottom && m_layout_search_alternate)
+	if (m_layout_search_alternate && m_layout_profile_alternate)
+	{
+		gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_contents_stack), 0);
+		gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_search_box), 1);
+		gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_title_box), 2);
+	}
+	else if (m_layout_profile_alternate)
+	{
+		gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_search_box), 0);
+		gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_contents_stack), 1);
+		gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_title_box), 2);
+	}
+	else if (m_layout_search_alternate)
 	{
 		gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_title_box), 0);
 		gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_contents_stack), 1);
 		gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_search_box), 2);
 	}
-	else if (m_layout_search_alternate)
-	{
-		gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_title_box), 2);
-		gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_contents_stack), 1);
-		gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_search_box), 0);
-	}
-	else if (m_layout_bottom)
+	else
 	{
 		gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_title_box), 0);
 		gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_search_box), 1);
 		gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_contents_stack), 2);
 	}
-	else
+
+	// Handle size group to category buttons
+	const bool category_show_name = wm_settings->category_show_name && !wm_settings->position_categories_horizontal;
+	if (!m_sidebar_size_group && m_layout_commands_alternate && category_show_name)
 	{
-		gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_title_box), 2);
-		gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_search_box), 1);
-		gtk_box_reorder_child(m_vbox, GTK_WIDGET(m_contents_stack), 0);
+		m_sidebar_size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
+		gtk_size_group_add_widget(m_sidebar_size_group, GTK_WIDGET(m_sidebar));
+		gtk_size_group_add_widget(m_sidebar_size_group, GTK_WIDGET(m_commands_box));
+	}
+	else if (m_sidebar_size_group && (!m_layout_commands_alternate || !category_show_name))
+	{
+		gtk_size_group_remove_widget(m_sidebar_size_group, GTK_WIDGET(m_sidebar));
+		gtk_size_group_remove_widget(m_sidebar_size_group, GTK_WIDGET(m_commands_box));
+		g_object_unref(m_sidebar_size_group);
+		m_sidebar_size_group = nullptr;
 	}
 }
 
